import { getSimpleDateFromString } from '../datelogic/simpledate';
import i18n from '../i18n/i18n';
/**
 * Generates month options for dropdown selection
 */
export var getMonthOptions = function (locale) {
    var months = [];
    for (var i = 1; i <= 12; i++) {
        var monthKey = "MONTH_".concat(i);
        months.push({
            value: i,
            label: i18n[locale][monthKey],
        });
    }
    return months;
};
/**
 * Generates year options based on min and max dates, or defaults to standard range
 */
export var getYearOptions = function (minDate, maxDate) {
    var currentYear = new Date().getFullYear();
    // Default range: 6 years backward and 2 years forward
    var minYear = currentYear - 6;
    var maxYear = currentYear + 2;
    // Use minDate year if provided
    if (minDate) {
        var minDateObj = getSimpleDateFromString(minDate);
        if (minDateObj) {
            minYear = minDateObj.year;
        }
    }
    // Use maxDate year if provided  
    if (maxDate) {
        var maxDateObj = getSimpleDateFromString(maxDate);
        if (maxDateObj) {
            maxYear = maxDateObj.year;
        }
    }
    var years = [];
    for (var i = minYear; i <= maxYear; i++) {
        years.push({
            value: i,
            label: String(i),
        });
    }
    return years;
};
