"use strict";
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.SimpleCalendar = void 0;
var i18n_1 = __importDefault(require("../i18n/i18n"));
var simpledate_1 = require("./simpledate");
var msInADay = 86400000;
var supportedLocales = ['nb', 'nn', 'en'];
function iso8601Week(date) {
    // Clone date
    var checkDate = date.clone();
    // Find Thursday of this week starting on Monday
    checkDate.date = date.date + 4 - (date.day || 7);
    // Copy time of Thursday of this week
    var time = checkDate.timestamp;
    // Adjust to Jan 1
    checkDate.month = 0;
    checkDate.date = 1;
    var daysSince1Jan = (time - checkDate.timestamp) / msInADay;
    // Return the correct weeknumber
    return Math.floor(Math.round(daysSince1Jan / 7) + 1);
}
function firstDayOfMonth(simpledate) {
    var clone = simpledate.clone();
    clone.date = 1;
    return clone.day;
}
var SimpleCalendar = /** @class */ (function () {
    function SimpleCalendar(initialDate, locale) {
        if (locale === void 0) { locale = 'nb'; }
        this.locale = locale;
        this.focusedDate = initialDate
            ? initialDate.clone()
            : (0, simpledate_1.getSimpleDateToday)();
        this.selectedDate = initialDate ? initialDate.clone() : initialDate;
        // Settings
        this.firstDay = i18n_1.default[locale].FIRST_DAY_OF_WEEK;
    }
    Object.defineProperty(SimpleCalendar.prototype, "focusedMonth", {
        get: function () {
            // @ts-ignore
            return i18n_1.default[this.locale]["MONTH_".concat(this.focusedDate.month + 1)];
        },
        enumerable: false,
        configurable: true
    });
    Object.defineProperty(SimpleCalendar.prototype, "focusedYear", {
        get: function () {
            return this.focusedDate.year;
        },
        enumerable: false,
        configurable: true
    });
    Object.defineProperty(SimpleCalendar.prototype, "dayNames", {
        get: function () {
            var result = [];
            for (var i = this.firstDay; i < this.firstDay + 7; i++) {
                result.push({
                    // @ts-ignore
                    name: i18n_1.default[this.locale]["DAY_".concat(i % 7)],
                    // @ts-ignore
                    shortName: i18n_1.default[this.locale]["DAY_".concat(i % 7, "_SHORT")],
                });
            }
            return result;
        },
        enumerable: false,
        configurable: true
    });
    Object.defineProperty(SimpleCalendar.prototype, "previousName", {
        get: function () {
            return i18n_1.default[this.locale].PREVIOUS_MONTH;
        },
        enumerable: false,
        configurable: true
    });
    Object.defineProperty(SimpleCalendar.prototype, "nextName", {
        get: function () {
            return i18n_1.default[this.locale].NEXT_MONTH;
        },
        enumerable: false,
        configurable: true
    });
    SimpleCalendar.prototype.previousYear = function () {
        this.focusedDate.adjust({ period: 'Y', offset: -1 });
    };
    SimpleCalendar.prototype.nextYear = function () {
        this.focusedDate.adjust({ period: 'Y', offset: 1 });
    };
    SimpleCalendar.prototype.nextMonth = function () {
        this.focusedDate.adjust({ period: 'M', offset: 1 });
    };
    SimpleCalendar.prototype.previousMonth = function () {
        this.focusedDate.adjust({ period: 'M', offset: -1 });
    };
    SimpleCalendar.prototype.nextWeek = function () {
        this.focusedDate.adjust({ period: 'D', offset: 7 });
    };
    SimpleCalendar.prototype.previousWeek = function () {
        this.focusedDate.adjust({ period: 'D', offset: -7 });
    };
    SimpleCalendar.prototype.nextDay = function () {
        this.focusedDate.adjust({ period: 'D', offset: 1 });
    };
    SimpleCalendar.prototype.previousDay = function () {
        this.focusedDate.adjust({ period: 'D', offset: -1 });
    };
    SimpleCalendar.prototype.firstDateOfMonth = function () {
        this.focusedDate.date = 1;
    };
    SimpleCalendar.prototype.lastDateOfMonth = function () {
        this.focusedDate.date = this.focusedDate.daysInMonth;
    };
    SimpleCalendar.prototype.selectFocusedDate = function () {
        this.selectedDate = this.focusedDate.clone();
    };
    SimpleCalendar.prototype.isDateWithinDateRange = function (date) {
        return true; // Always allow all dates
    };
    SimpleCalendar.prototype.selectTimestamp = function (timestamp) {
        this.focusedDate = (0, simpledate_1.getSimpleDateFromTimestamp)(timestamp);
        this.selectFocusedDate();
    };
    SimpleCalendar.prototype.selected = function () {
        return this.selectedDate ? this.selectedDate.format() : '';
    };
    SimpleCalendar.prototype.focused = function () {
        return this.focusedDate ? this.focusedDate.format() : '';
    };
    SimpleCalendar.prototype.selectDateString = function (date) {
        var newDate = (0, simpledate_1.getSimpleDateFromString)(date);
        if (!newDate) {
            return;
        }
        this.selectedDate = newDate.clone();
        this.focusedDate = newDate.clone();
    };
    Object.defineProperty(SimpleCalendar.prototype, "visibleDates", {
        get: function () {
            var result = [];
            var leadDays = (firstDayOfMonth(this.focusedDate) - this.firstDay + 7) % 7;
            var currentDate = this.focusedDate.clone();
            currentDate.date = 1 - leadDays;
            var rows = Math.ceil((leadDays + this.focusedDate.daysInMonth) / 7);
            for (var row = 0; row < rows; row++) {
                var week = {
                    number: iso8601Week(currentDate),
                    dates: [],
                };
                result.push(week);
                for (var dayx = 0; dayx < 7; dayx++) {
                    var date = {
                        dayInMonth: currentDate.date,
                        timestamp: currentDate.timestamp,
                        isNonClickableDate: currentDate.month !== this.focusedDate.month,
                        isToday: currentDate.equal((0, simpledate_1.getSimpleDateToday)()),
                        isFocus: currentDate.equal(this.focusedDate),
                        isSelected: !!this.selectedDate &&
                            currentDate.equal(this.selectedDate),
                        isEnabled: true, // Always enable all dates
                    };
                    week.dates.push(date);
                    currentDate.adjust({ period: 'D', offset: 1 });
                }
            }
            return result;
        },
        enumerable: false,
        configurable: true
    });
    return SimpleCalendar;
}());
exports.SimpleCalendar = SimpleCalendar;
