"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.getSimpleDateFromString = exports.getSimpleDateFromTimestamp = exports.getSimpleDateToday = exports.SimpleDate = void 0;
var dateUtil_1 = require("../util/dateUtil");
function isDate(value) {
    return (!!value &&
        typeof value === 'object' &&
        Object.prototype.toString.call(value) === '[object Date]');
}
function daysInMonth(year, month) {
    return 32 - new Date(year, month, 32).getDate();
}
var SimpleDate = /** @class */ (function () {
    function SimpleDate(date) {
        if (isDate(date)) {
            this.internalDate = date;
        }
        else {
            this.internalDate = new Date();
        }
    }
    Object.defineProperty(SimpleDate.prototype, "day", {
        get: function () {
            return this.internalDate.getDay();
        },
        enumerable: false,
        configurable: true
    });
    Object.defineProperty(SimpleDate.prototype, "date", {
        get: function () {
            return this.internalDate.getDate();
        },
        set: function (date) {
            this.internalDate = new Date(this.year, this.month, date);
        },
        enumerable: false,
        configurable: true
    });
    Object.defineProperty(SimpleDate.prototype, "month", {
        get: function () {
            return this.internalDate.getMonth();
        },
        set: function (month) {
            this.internalDate = new Date(this.year, month, this.date);
        },
        enumerable: false,
        configurable: true
    });
    Object.defineProperty(SimpleDate.prototype, "year", {
        get: function () {
            return this.internalDate.getFullYear();
        },
        set: function (year) {
            this.internalDate = new Date(year, this.month, this.date);
        },
        enumerable: false,
        configurable: true
    });
    Object.defineProperty(SimpleDate.prototype, "daysInMonth", {
        get: function () {
            return daysInMonth(this.year, this.month);
        },
        enumerable: false,
        configurable: true
    });
    Object.defineProperty(SimpleDate.prototype, "timestamp", {
        get: function () {
            return this.internalDate.getTime();
        },
        enumerable: false,
        configurable: true
    });
    SimpleDate.prototype.adjust = function (_a) {
        var period = _a.period, offset = _a.offset;
        var date = period === 'D' ? this.date + offset : this.date;
        var month = period === 'M' ? this.month + offset : this.month;
        var year = period === 'Y' ? this.year + offset : this.year;
        if (period !== 'D') {
            date = Math.max(1, Math.min(date, daysInMonth(year, month)));
        }
        this.internalDate = new Date(year, month, date);
    };
    SimpleDate.prototype.equal = function (other) {
        return (this.date === other.date &&
            this.month === other.month &&
            this.year === other.year);
    };
    SimpleDate.prototype.isBefore = function (other) {
        return this.timestamp < other.timestamp;
    };
    SimpleDate.prototype.isAfter = function (other) {
        return this.timestamp > other.timestamp;
    };
    SimpleDate.prototype.clone = function () {
        var date = new Date(this.year, this.month, this.date);
        return new SimpleDate(date);
    };
    SimpleDate.prototype.format = function () {
        var day = this.date < 10 ? "0".concat(this.date) : this.date;
        var normalizedMonthNumber = this.month + 1;
        var month = normalizedMonthNumber < 10
            ? "0".concat(normalizedMonthNumber)
            : normalizedMonthNumber;
        return "".concat(day, ".").concat(month, ".").concat(this.year);
    };
    SimpleDate.prototype.toString = function () {
        return this.format();
    };
    return SimpleDate;
}());
exports.SimpleDate = SimpleDate;
var getSimpleDateToday = function () { return new SimpleDate(); };
exports.getSimpleDateToday = getSimpleDateToday;
var getSimpleDateFromTimestamp = function (timestamp) {
    return new SimpleDate(new Date(timestamp));
};
exports.getSimpleDateFromTimestamp = getSimpleDateFromTimestamp;
function beforeYear(year, yearLimit) {
    return year < yearLimit;
}
/*
  If given a one- or two-digit year we assume this century.
  E.g: 01.01.17 will yield 01.01.2017, not 01.01.1917 (JS default)
  Else they'll have to use 4-digit year.
*/
function deriveOneOrTwoDigitYear(year) {
    var yearDate = new Date(year, 0);
    var fullYear = yearDate.getFullYear();
    if (beforeYear(fullYear, 2000)) {
        return yearDate.getFullYear() + 100;
    }
    return yearDate.getFullYear();
}
/*
  If given a three-digit year we assume this millennium.
  E.g: 0101017 will yield 01.01.2017, or 0101117 01.01.2117
  Else they'll have to use 4-digit year.
*/
function deriveThreeDigitYear(year) {
    return parseInt("2".concat(year), 10);
}
var noop = function () { };
/*ts ignore just for keeing old js implementation*/
var getSimpleDateFromString = function (value, onSuccess) {
    if (onSuccess === void 0) { onSuccess = noop; }
    var match = value && (0, dateUtil_1.validateDate)(value);
    if (!match) {
        return null;
    }
    var date = match[1];
    var month = match[3];
    var year = match[4];
    if ((year.length === 1 && year !== '0') || year.length === 2) {
        // @ts-ignore
        year = deriveOneOrTwoDigitYear(year);
    }
    if (year.length === 3) {
        // @ts-ignore
        year = deriveThreeDigitYear(year);
    }
    // If no year and/or month is input, use todays month and/or year
    var today = new Date();
    if (!month) {
        // @ts-ignore
        month = today.getMonth() + 1;
    }
    if (!year || year === '0') {
        // @ts-ignore
        year = today.getFullYear();
    }
    // @ts-ignore
    var newDate = new SimpleDate(new Date(year, month - 1, date));
    if (
    // @ts-ignore
    newDate.date !== date * 1 ||
        // @ts-ignore
        newDate.month + 1 !== month * 1 ||
        // @ts-ignore
        newDate.year !== year * 1) {
        return null;
    }
    onSuccess(newDate);
    return newDate;
};
exports.getSimpleDateFromString = getSimpleDateFromString;
