"use strict";
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.getYearOptions = exports.getMonthOptions = void 0;
var simpledate_1 = require("../datelogic/simpledate");
var i18n_1 = __importDefault(require("../i18n/i18n"));
/**
 * Generates month options for dropdown selection
 */
var getMonthOptions = function (locale) {
    var months = [];
    for (var i = 1; i <= 12; i++) {
        var monthKey = "MONTH_".concat(i);
        months.push({
            value: i,
            label: i18n_1.default[locale][monthKey],
        });
    }
    return months;
};
exports.getMonthOptions = getMonthOptions;
/**
 * Generates year options based on min and max dates, or defaults to standard range
 */
var getYearOptions = function (minDate, maxDate) {
    var currentYear = new Date().getFullYear();
    // Default range: 6 years backward and 2 years forward
    var minYear = currentYear - 6;
    var maxYear = currentYear + 2;
    // Use minDate year if provided
    if (minDate) {
        var minDateObj = (0, simpledate_1.getSimpleDateFromString)(minDate);
        if (minDateObj) {
            minYear = minDateObj.year;
        }
    }
    // Use maxDate year if provided  
    if (maxDate) {
        var maxDateObj = (0, simpledate_1.getSimpleDateFromString)(maxDate);
        if (maxDateObj) {
            maxYear = maxDateObj.year;
        }
    }
    var years = [];
    for (var i = minYear; i <= maxYear; i++) {
        years.push({
            value: i,
            label: String(i),
        });
    }
    return years;
};
exports.getYearOptions = getYearOptions;
