import React, { Component } from 'react';
import { SimpleCalendar } from '../datelogic/simplecalendar';
import { CalendarButtonState } from '../datelogic/types';
export interface CalendarProps {
    calendarClassName?: string;
    escKeyHandler?: React.KeyboardEventHandler<HTMLDivElement>;
    locale: 'nb' | 'nn' | 'en';
    onDatePicked: (date: string) => void;
    selectedDate?: string | null;
    focusOnMount?: boolean;
    /** Om måned- og år-dropdown skal vises i kalenderen */
    dropdownCaption?: boolean;
    /** Tidligste tillatte dato (format: 'dd.mm.yyyy') - brukes kun til å bestemme år-intervall i dropdown */
    minDate?: string | null;
    /** Seneste tillatte dato (format: 'dd.mm.yyyy') - brukes kun til å bestemme år-intervall i dropdown */
    maxDate?: string | null;
}
interface State {
    calendar: SimpleCalendar;
    isFocusingHeader: boolean;
}
export declare class Calendar extends Component<CalendarProps, State> {
    private readonly datepickerId;
    constructor(props: CalendarProps);
    clickableDateRef: React.RefObject<HTMLTableCellElement>;
    prevMonthButtonElementRef: React.RefObject<HTMLButtonElement>;
    nextMonthButtonElementRef: React.RefObject<HTMLButtonElement>;
    componentDidUpdate(prevProps: CalendarProps): void;
    shouldComponentUpdate(nextProps: CalendarProps): boolean;
    keyDown(event: React.KeyboardEvent<HTMLTableElement>): void;
    mouseClick(date: CalendarButtonState): void;
    nextMonth(evt: React.MouseEvent<HTMLButtonElement>): void;
    previousMonth(evt: React.MouseEvent<HTMLButtonElement>): void;
    renderDate(calendarButtonState: CalendarButtonState, index: number): React.JSX.Element;
    renderWeek(week: {
        dates: CalendarButtonState[];
        number: number;
    }): React.JSX.Element;
    renderDay(day: {
        name: string;
        shortName: string;
    }, index: number): React.JSX.Element;
    focusTrap: (event: React.KeyboardEvent<HTMLDivElement>) => void;
    /**
     * Navigate to a specific month and year
     */
    navigateToMonthYear(month: number, year: number): void;
    render(): React.JSX.Element;
}
export {};
